# utils.py

import os
import re
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from parameters import titles, NS_TO_US

# Function to extract total simulation time from log file
def get_total_sim_time(log_path):
    """ Extracts the total simulation time from a given log file """
    with open(log_path, "r") as file:
        lines = file.readlines()

    last_time_ns = None
    for line in reversed(lines):  # Read from the end for efficiency
        match = re.search(r"(\d+) ns: INFO: MY_LOG:", line)
        if match:
            last_time_ns = int(match.group(1))
            break

    return round(last_time_ns / NS_TO_US, 2) if last_time_ns else None

# Function to calculate throughput
def get_throughput(log_path, sim_time):
    """ Calculates throughput as packets per microsecond """
    if sim_time is None:
        return None

    last_packet_id = None
    with open(log_path, "r") as file:
        for line in reversed(file.readlines()):  # Read from the end
            match = re.search(r"Packet with id (\d+) arrived", line)
            if match:
                last_packet_id = int(match.group(1))
                break
    last_packet_id = last_packet_id + 1  # Total packets processed
    return round(last_packet_id / sim_time, 2) if last_packet_id else None

# Function to generate the simulation time graph
def generate_sim_time_graph(sim_time_table, model, delay):
    """ Plots total simulation time for the given model and delay setting """
    plt.figure(figsize=(8, 5))
    ax = sns.barplot(data=sim_time_table, x="architecture", y="simulation time (us)", hue="architecture", 
                     palette="Blues", width=0.4, legend=False)
    ax.set_title(f"Total Simulation Time ({titles.get(model, model)}) - {delay} Delay")
    ax.set_ylabel("Simulation Time (us)")

    # Label bars
    for container in ax.containers:
        ax.bar_label(container, fontsize=10)

    plt.savefig(f"simulation_time_{model}_{delay}.png")
    plt.savefig(f"simulation_time_{model}_{delay}.pdf")
    plt.clf()

# Function to generate the throughput graph
def generate_throughput_graph(sim_time_table, model, delay):
    """ Plots throughput for the given model and delay setting """
    plt.figure(figsize=(8, 5))
    ax = sns.barplot(data=sim_time_table, x="architecture", y="throughput (packets/us)", hue="architecture", 
                     palette="Reds", width=0.4, legend=False)
    ax.set_title(f"Throughput ({titles.get(model, model)}) - {delay} Delay")
    ax.set_ylabel("Throughput (packets/us)")

    # Label bars
    for container in ax.containers:
        ax.bar_label(container, fontsize=10)

    plt.savefig(f"throughput_{model}_{delay}.png")
    plt.savefig(f"throughput_{model}_{delay}.pdf")
    plt.clf()
